@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2009 Fabrice Colin
+ *  Copyright 2005-2012 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,14 +37,15 @@
 #include "StringManip.h"
 #include "TimeConverter.h"
 #include "Url.h"
+#include "FieldMapperInterface.h"
 #include "LanguageDetector.h"
 #include "XapianDatabaseFactory.h"
 #include "XapianIndex.h"
 
 #define MAGIC_TERM "X-MetaSE-Doc"
 
-using std::cout;
-using std::cerr;
+using std::clog;
+using std::clog;
 using std::endl;
 using std::ios;
 using std::ifstream;
@@ -55,6 +56,9 @@ using std::set;
 using std::map;
 using std::min;
 using std::max;
+using std::pair;
+
+extern FieldMapperInterface *g_pMapper;
 
 static string removeCharsetFromType(const string &type)
 {
@@ -192,9 +196,6 @@ class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 					string::size_type startPos = 0;
 					bool addRemainder = true;
 
-#ifdef DEBUG
-					cout << "TokensIndexer::handle_token: found dots in " << term << endl;
-#endif
 					while (dotPos != string::npos)
 					{
 						string component(term.substr(startPos, dotPos - startPos));
@@ -249,7 +250,7 @@ class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 				}
 				catch (const Xapian::UnimplementedError &error)
 				{
-					cerr << "Couldn't index with spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
+					clog << "Couldn't index with spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
 
 					m_doSpelling = false;
 				}
@@ -322,7 +323,7 @@ bool XapianIndex::listDocumentsWithTerm(const string &term, set<unsigned int> &d
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -333,7 +334,7 @@ bool XapianIndex::listDocumentsWithTerm(const string &term, set<unsigned int> &d
 		if (pIndex != NULL)
 		{
 #ifdef DEBUG
-			cout << "XapianIndex::listDocumentsWithTerm: term " << term << endl;
+			clog << "XapianIndex::listDocumentsWithTerm: term " << term << endl;
 #endif
 			// Get a list of documents that have the term
 			for (Xapian::PostingIterator postingIter = pIndex->postlist_begin(term);
@@ -354,11 +355,11 @@ bool XapianIndex::listDocumentsWithTerm(const string &term, set<unsigned int> &d
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get document list: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get document list: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get document list, unknown exception occured" << endl;
+		clog << "Couldn't get document list, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -382,7 +383,7 @@ void XapianIndex::addPostingsToDocument(const Xapian::Utf8Iterator &itor, Xapian
 		}
 		catch (const Xapian::Error &error)
 		{
-			cerr << "Couldn't create stemmer: " << error.get_type() << ": " << error.get_msg() << endl;
+			clog << "Couldn't create stemmer: " << error.get_type() << ": " << error.get_msg() << endl;
 		}
 	}
 
@@ -431,7 +432,7 @@ void XapianIndex::addPostingsToDocument(const Xapian::Utf8Iterator &itor, Xapian
 		}
 		catch (const Xapian::UnimplementedError &error)
 		{
-			cerr << "Couldn't index with spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
+			clog << "Couldn't index with spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
 
 			if (doSpelling == true)
 			{
@@ -464,7 +465,7 @@ void XapianIndex::addPostingsToDocument(Dijon::CJKVTokenizer &tokenizer, Xapian:
 	// Get the terms
 	tokenizer.tokenize(text, handler, true);
 #ifdef DEBUG
-	cout << "XapianIndex::addPostingsToDocument: terms to position " << termPos << endl;
+	clog << "XapianIndex::addPostingsToDocument: terms to position " << termPos << endl;
 #endif
 }
 
@@ -489,7 +490,7 @@ void XapianIndex::addLabelsToDocument(Xapian::Document &doc, const set<string> &
 		}
 
 #ifdef DEBUG
-		cout << "XapianIndex::addLabelsToDocument: label \"" << labelName << "\"" << endl;
+		clog << "XapianIndex::addLabelsToDocument: label \"" << labelName << "\"" << endl;
 #endif
 		doc.add_term(string("XLABEL:") + XapianDatabase::limitTermLength(Url::escapeUrl(labelName)));
 	}
@@ -515,7 +516,7 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 		bool removeTerm = false;
 
 #ifdef DEBUG
-		cout << "XapianIndex::removePostingsFromDocument: term " << *termListIter
+		clog << "XapianIndex::removePostingsFromDocument: term " << *termListIter
 			<< " has " << postingsCount << " postings" << endl;
 #endif
 		// If a prefix is defined, or there are no postings, we can afford removing the term
@@ -537,7 +538,7 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 					{
 						// This term doesn't exist in the document !
 #ifdef DEBUG
-						cout << "XapianIndex::removePostingsFromDocument: no such term" << endl;
+						clog << "XapianIndex::removePostingsFromDocument: no such term" << endl;
 #endif
 						continue;
 					}
@@ -546,7 +547,7 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 					{
 						// All postings are to be removed, so we can remove the term
 #ifdef DEBUG
-						cout << "XapianIndex::removePostingsFromDocument: no extra posting" << endl;
+						clog << "XapianIndex::removePostingsFromDocument: no extra posting" << endl;
 #endif
 						removeTerm = true;
 					}
@@ -563,7 +564,7 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 			catch (const Xapian::Error &error)
 			{
 #ifdef DEBUG
-				cout << "XapianIndex::removePostingsFromDocument: " << error.get_msg() << endl;
+				clog << "XapianIndex::removePostingsFromDocument: " << error.get_msg() << endl;
 #endif
 			}
 
@@ -577,13 +578,13 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 			}
 			catch (const Xapian::UnimplementedError &error)
 			{
-				cerr << "Couldn't remove spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
+				clog << "Couldn't remove spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
 				doSpelling = false;
 			}
 			catch (const Xapian::Error &error)
 			{
 #ifdef DEBUG
-				cout << "XapianIndex::removePostingsFromDocument: " << error.get_msg() << endl;
+				clog << "XapianIndex::removePostingsFromDocument: " << error.get_msg() << endl;
 #endif
 			}
 			continue;
@@ -609,19 +610,19 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 			{
 				// This posting may have been removed already
 #ifdef DEBUG
-				cout << "XapianIndex::removePostingsFromDocument: " << error.get_msg() << endl;
+				clog << "XapianIndex::removePostingsFromDocument: " << error.get_msg() << endl;
 #endif
 			}
 		}
 	}
 }
 
-void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc,
+void XapianIndex::addCommonTerms(const DocumentInfo &docInfo, Xapian::Document &doc,
 	const Xapian::WritableDatabase &db, Xapian::termcount &termPos)
 {
-	string title(info.getTitle());
-	string location(info.getLocation());
-	string type(removeCharsetFromType(info.getType()));
+	string title(docInfo.getTitle());
+	string location(docInfo.getLocation());
+	string type(removeCharsetFromType(docInfo.getType()));
 	Url urlObj(location);
 
 	// Add a magic term :-)
@@ -635,10 +636,10 @@ void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc
 	}
 
 	// Index the full URL with prefix U
-	doc.add_term(string("U") + XapianDatabase::limitTermLength(Url::escapeUrl(info.getLocation(true)), true));
+	doc.add_term(string("U") + XapianDatabase::limitTermLength(Url::escapeUrl(docInfo.getLocation(true)), true));
 	// ...the base file with XFILE:
 	if ((urlObj.isLocal() == true) &&
-		(info.getInternalPath().empty() == false))
+		(docInfo.getInternalPath().empty() == false))
 	{
 		string protocol(urlObj.getProtocol());
 
@@ -719,7 +720,7 @@ void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc
 		}
 		doc.add_term(string("XEXT:") + XapianDatabase::limitTermLength(extension));
 	}
-	// Finally, add the language code with prefix L
+	// Add the language code with prefix L
 	doc.add_term(string("L") + Languages::toCode(m_stemLanguage));
 	// ...and the MIME type with prefix T
 	doc.add_term(string("T") + type);
@@ -728,6 +729,19 @@ void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc
 	{
 		doc.add_term(string("XCLASS:") + type.substr(0, slashPos));
 	}
+	// Others
+	if (g_pMapper != NULL)
+	{
+		vector<pair<string, string> > prefixedTerms;
+
+		g_pMapper->getTerms(docInfo, prefixedTerms);
+
+		for (vector<pair<string, string> >::const_iterator termIter = prefixedTerms.begin();
+			termIter != prefixedTerms.end(); ++termIter)
+		{
+			doc.add_term(termIter->second + XapianDatabase::limitTermLength(termIter->first));
+		}
+	}
 }
 
 void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::WritableDatabase &db)
@@ -776,7 +790,7 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 
 			// Add another term with file as protocol
 			fileUrl.replace(0, protocol.length(), "file");
-			doc.add_term(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(fileUrl), true));
+			commonTerms.insert(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(fileUrl), true));
 		}
 	}
 	// Host name
@@ -854,6 +868,19 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 	{
 		commonTerms.insert(string("XCLASS:") + type.substr(0, slashPos));
 	}
+	// Others
+	if (g_pMapper != NULL)
+	{
+		vector<pair<string, string> > prefixedTerms;
+
+		g_pMapper->getTerms(docInfo, prefixedTerms);
+
+		for (vector<pair<string, string> >::const_iterator termIter = prefixedTerms.begin();
+			termIter != prefixedTerms.end(); ++termIter)
+		{
+			commonTerms.insert(termIter->second + XapianDatabase::limitTermLength(termIter->first));
+		}
+	}
 
 	for (set<string>::const_iterator termIter = commonTerms.begin(); termIter != commonTerms.end(); ++termIter)
 	{
@@ -864,7 +891,7 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 		catch (const Xapian::Error &error)
 		{
 #ifdef DEBUG
-			cout << "XapianIndex::removeCommonTerms: " << error.get_msg() << endl;
+			clog << "XapianIndex::removeCommonTerms: " << error.get_msg() << endl;
 #endif
 		}
 	}
@@ -906,7 +933,7 @@ string XapianIndex::scanDocument(const string &suggestedLanguage,
 		}
 		catch (const Xapian::Error &error)
 		{
-			cerr << "Invalid language: " << error.get_type() << ": " << error.get_msg() << endl;
+			clog << "Invalid language: " << error.get_type() << ": " << error.get_msg() << endl;
 
 			if (scannedDocument == false)
 			{
@@ -928,16 +955,16 @@ string XapianIndex::scanDocument(const string &suggestedLanguage,
 		break;
 	}
 #ifdef DEBUG
-	cout << "XapianIndex::scanDocument: language " << language << endl;
+	clog << "XapianIndex::scanDocument: language " << language << endl;
 #endif
 
 	return language;
 }
 
-void XapianIndex::setDocumentData(const DocumentInfo &info, Xapian::Document &doc,
+void XapianIndex::setDocumentData(const DocumentInfo &docInfo, Xapian::Document &doc,
 	const string &language) const
 {
-	time_t timeT = TimeConverter::fromTimestamp(info.getTimestamp());
+	time_t timeT = TimeConverter::fromTimestamp(docInfo.getTimestamp());
 	struct tm *tm = localtime(&timeT);
 	string yyyymmdd(TimeConverter::toYYYYMMDDString(tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday));
 	string hhmmss(TimeConverter::toHHMMSSString(tm->tm_hour, tm->tm_min, tm->tm_sec));
@@ -946,13 +973,25 @@ void XapianIndex::setDocumentData(const DocumentInfo &info, Xapian::Document &do
 	doc.add_value(0, yyyymmdd);
 	// FIXME: checksum in value 1
 	// Size
-	doc.add_value(2, Xapian::sortable_serialise((double )info.getSize()));
+	doc.add_value(2, Xapian::sortable_serialise((double )docInfo.getSize()));
 	// Time
 	doc.add_value(3, hhmmss);
 	// Date and time, for results sorting
 	doc.add_value(4, yyyymmdd + hhmmss);
+	// Any custom value ?
+	if (g_pMapper != NULL)
+	{
+		map<unsigned int, string> values;
 
-	DocumentInfo docCopy(info);
+		g_pMapper->getValues(docInfo, values);
+		for (map<unsigned int, string>::const_iterator valIter = values.begin();
+			valIter != values.end(); ++valIter)
+		{
+			doc.add_value(valIter->first, valIter->second);
+		}
+	}
+
+	DocumentInfo docCopy(docInfo);
 	// XapianDatabase expects the language in English, which is okay here
 	docCopy.setLanguage(language);
 	doc.set_data(XapianDatabase::propsToRecord(&docCopy));
@@ -970,7 +1009,7 @@ bool XapianIndex::deleteDocuments(const string &term)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -980,7 +1019,7 @@ bool XapianIndex::deleteDocuments(const string &term)
 		if (pIndex != NULL)
 		{
 #ifdef DEBUG
-			cout << "XapianIndex::deleteDocuments: term is " << term << endl;
+			clog << "XapianIndex::deleteDocuments: term is " << term << endl;
 #endif
 
 			// Delete documents from the index
@@ -991,11 +1030,11 @@ bool XapianIndex::deleteDocuments(const string &term)
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't unindex documents: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't unindex documents: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't unindex documents, unknown exception occured" << endl;
+		clog << "Couldn't unindex documents, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1021,7 +1060,7 @@ string XapianIndex::getMetadata(const string &name) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return "";
 	}
 
@@ -1037,11 +1076,11 @@ string XapianIndex::getMetadata(const string &name) const
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get metadata: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get metadata: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get metadata, unknown exception occured" << endl;
+		clog << "Couldn't get metadata, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1060,7 +1099,7 @@ bool XapianIndex::setMetadata(const string &name, const string &value) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1075,15 +1114,15 @@ bool XapianIndex::setMetadata(const string &name, const string &value) const
 	}
 	catch (const Xapian::UnimplementedError &error)
 	{
-		cerr << "Couldn't set metadata: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't set metadata: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't set metadata: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't set metadata: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't set metadata, unknown exception occured" << endl;
+		clog << "Couldn't set metadata, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1112,7 +1151,7 @@ bool XapianIndex::getDocumentInfo(unsigned int docId, DocumentInfo &docInfo) con
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1136,11 +1175,11 @@ bool XapianIndex::getDocumentInfo(unsigned int docId, DocumentInfo &docInfo) con
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get document properties: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get document properties: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get document properties, unknown exception occured" << endl;
+		clog << "Couldn't get document properties, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1155,7 +1194,7 @@ unsigned int XapianIndex::getDocumentTermsCount(unsigned int docId) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1168,17 +1207,17 @@ unsigned int XapianIndex::getDocumentTermsCount(unsigned int docId) const
 
 			termsCount = doc.termlist_count();
 #ifdef DEBUG
-			cout << "XapianIndex::getDocumentTermsCount: " << termsCount << " terms in document " << docId << endl;
+			clog << "XapianIndex::getDocumentTermsCount: " << termsCount << " terms in document " << docId << endl;
 #endif
 		}
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get document terms count: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get document terms count: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get document terms count, unknown exception occured" << endl;
+		clog << "Couldn't get document terms count, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1194,7 +1233,7 @@ bool XapianIndex::getDocumentTerms(unsigned int docId, map<unsigned int, string>
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1220,7 +1259,7 @@ bool XapianIndex::getDocumentTerms(unsigned int docId, map<unsigned int, string>
 					if (firstChar == 'X')
 					{
 #ifdef DEBUG
-						cout << "XapianIndex::getDocumentTerms: skipping " << termName << endl;
+						clog << "XapianIndex::getDocumentTerms: skipping " << termName << endl;
 #endif
 						continue;
 					}
@@ -1248,7 +1287,7 @@ bool XapianIndex::getDocumentTerms(unsigned int docId, map<unsigned int, string>
 				gotTerms = true;
 			}
 
-			// Append terms without positional information as if they were at the end of the document
+			// Append terms without positional docInformation as if they were at the end of the document
 			for (vector<string>::const_iterator noPosIter = noPosTerms.begin();
 				noPosIter != noPosTerms.end(); ++noPosIter)
 			{
@@ -1259,11 +1298,11 @@ bool XapianIndex::getDocumentTerms(unsigned int docId, map<unsigned int, string>
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get document terms: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get document terms: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get document terms, unknown exception occured" << endl;
+		clog << "Couldn't get document terms, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1341,7 +1380,7 @@ bool XapianIndex::deleteLabel(const string &name)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1371,11 +1410,11 @@ bool XapianIndex::deleteLabel(const string &name)
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't delete label: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't delete label: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't delete label, unknown exception occured" << endl;
+		clog << "Couldn't delete label, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1390,7 +1429,7 @@ bool XapianIndex::hasLabel(unsigned int docId, const string &name) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1419,11 +1458,11 @@ bool XapianIndex::hasLabel(unsigned int docId, const string &name) const
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't check document labels: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't check document labels: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't check document labels, unknown exception occured" << endl;
+		clog << "Couldn't check document labels, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1438,7 +1477,7 @@ bool XapianIndex::getDocumentLabels(unsigned int docId, set<string> &labels) con
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1471,11 +1510,11 @@ bool XapianIndex::getDocumentLabels(unsigned int docId, set<string> &labels) con
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get document's labels: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get document's labels: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get document's labels, unknown exception occured" << endl;
+		clog << "Couldn't get document's labels, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1501,7 +1540,7 @@ bool XapianIndex::setDocumentsLabels(const set<unsigned int> &docIds,
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1548,11 +1587,11 @@ bool XapianIndex::setDocumentsLabels(const set<unsigned int> &docIds,
 		}
 		catch (const Xapian::Error &error)
 		{
-			cerr << "Couldn't update document's labels: " << error.get_type() << ": " << error.get_msg() << endl;
+			clog << "Couldn't update document's labels: " << error.get_type() << ": " << error.get_msg() << endl;
 		}
 		catch (...)
 		{
-			cerr << "Couldn't update document's labels, unknown exception occured" << endl;
+			clog << "Couldn't update document's labels, unknown exception occured" << endl;
 		}
 
 		pDatabase->unlock();
@@ -1569,7 +1608,7 @@ unsigned int XapianIndex::hasDocument(const string &url) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1587,7 +1626,7 @@ unsigned int XapianIndex::hasDocument(const string &url) const
 				// This URL was indexed
 				docId = *postingIter;
 #ifdef DEBUG
-				cout << "XapianIndex::hasDocument: " << term << " in document "
+				clog << "XapianIndex::hasDocument: " << term << " in document "
 					<< docId << " " << postingIter.get_wdf() << " time(s)" << endl;
 #endif
 			}
@@ -1596,11 +1635,11 @@ unsigned int XapianIndex::hasDocument(const string &url) const
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't look for document: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't look for document: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't look for document, unknown exception occured" << endl;
+		clog << "Couldn't look for document, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1621,7 +1660,7 @@ unsigned int XapianIndex::getCloseTerms(const string &term, set<string> &suggest
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1658,11 +1697,11 @@ unsigned int XapianIndex::getCloseTerms(const string &term, set<string> &suggest
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get terms: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get terms: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get terms, unknown exception occured" << endl;
+		clog << "Couldn't get terms, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1677,7 +1716,7 @@ unsigned int XapianIndex::getLastDocumentID(void) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1691,11 +1730,11 @@ unsigned int XapianIndex::getLastDocumentID(void) const
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get last document ID: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't get last document ID: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get last document ID, unknown exception occured" << endl;
+		clog << "Couldn't get last document ID, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1710,7 +1749,7 @@ unsigned int XapianIndex::getDocumentsCount(const string &labelName) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1736,11 +1775,11 @@ unsigned int XapianIndex::getDocumentsCount(const string &labelName) const
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't count documents: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't count documents: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't count documents, unknown exception occured" << endl;
+		clog << "Couldn't count documents, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1786,7 +1825,7 @@ bool XapianIndex::indexDocument(const Document &document, const std::set<std::st
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1824,7 +1863,7 @@ bool XapianIndex::indexDocument(const Document &document, const std::set<std::st
 					false, m_doSpelling, termPos);
 			}
 #ifdef DEBUG
-			cout << "XapianIndex::indexDocument: " << labels.size() << " labels for URL " << docInfo.getLocation(true) << endl;
+			clog << "XapianIndex::indexDocument: " << labels.size() << " labels for URL " << docInfo.getLocation(true) << endl;
 #endif
 
 			// Add labels
@@ -1840,11 +1879,11 @@ bool XapianIndex::indexDocument(const Document &document, const std::set<std::st
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't index document: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't index document: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't index document, unknown exception occured" << endl;
+		clog << "Couldn't index document, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1859,7 +1898,7 @@ bool XapianIndex::updateDocument(unsigned int docId, const Document &document)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1917,11 +1956,11 @@ bool XapianIndex::updateDocument(unsigned int docId, const Document &document)
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't update document: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't update document: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't update document, unknown exception occured" << endl;
+		clog << "Couldn't update document, unknown exception occured" << endl;
 	}
 	if (pIndex != NULL)
 	{
@@ -1944,7 +1983,7 @@ bool XapianIndex::updateDocumentInfo(unsigned int docId, const DocumentInfo &doc
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1968,11 +2007,11 @@ bool XapianIndex::updateDocumentInfo(unsigned int docId, const DocumentInfo &doc
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't update document properties: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't update document properties: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't update document properties, unknown exception occured" << endl;
+		clog << "Couldn't update document properties, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -1992,7 +2031,7 @@ bool XapianIndex::unindexDocument(unsigned int docId)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -2008,11 +2047,11 @@ bool XapianIndex::unindexDocument(unsigned int docId)
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't unindex document: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't unindex document: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't unindex document, unknown exception occured" << endl;
+		clog << "Couldn't unindex document, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -2063,7 +2102,7 @@ bool XapianIndex::flush(void)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -2078,11 +2117,11 @@ bool XapianIndex::flush(void)
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't flush database: " << error.get_type() << ": " << error.get_msg() << endl;
+		clog << "Couldn't flush database: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't flush database, unknown exception occured" << endl;
+		clog << "Couldn't flush database, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
 
@@ -2096,7 +2135,7 @@ bool XapianIndex::reopen(void) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 	pDatabase->reopen();
@@ -2111,7 +2150,7 @@ bool XapianIndex::reset(void)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false, true);
 	if (pDatabase == NULL)
 	{
-		cerr << "Couldn't get index " << m_databaseName << endl;
+		clog << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
