@@ -1,6 +1,6 @@
 /*
  *  Copyright 2007-2008 林永忠 Yung-Chung Lin
- *  Copyright 2008-2009 Fabrice Colin
+ *  Copyright 2008-2011 Fabrice Colin
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,7 @@
 #ifdef HAVE_UNICODE_H
 #include <unicode.h>
 #else
-#include <glib/gunicode.h>
+#include <glib.h>
 #define unicode_char_t gunichar
 #endif
 
