@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2010 Fabrice Colin
+ *  Copyright 2007-2012 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,19 +30,17 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <iostream>
-#include <utility>
-#include <algorithm>
 
 #include <gmime/gmime.h>
 
 #include "GMimeMboxFilter.h"
 
-using std::cout;
-using std::cerr;
+using std::clog;
+using std::clog;
 using std::endl;
 using std::string;
 using std::map;
-using std::max;
+using std::pair;
 using namespace Dijon;
 
 #ifdef _DYNAMIC_DIJON_FILTERS
@@ -82,9 +80,6 @@ DIJON_FILTER_INITIALIZE void initialize_gmime(void)
 #else
 	g_mime_init(GMIME_INIT_FLAG_UTF8);
 #endif
-#ifdef DEBUG
-	cout << "GMimeMboxFilter: initialized" << endl;
-#endif
 }
 
 DIJON_FILTER_SHUTDOWN void shutdown_gmime(void)
@@ -94,6 +89,50 @@ DIJON_FILTER_SHUTDOWN void shutdown_gmime(void)
 }
 #endif
 
+static string extractField(const string &str,
+	const string &start, const string &end,
+	string::size_type &endPos, bool anyCharacterOfEnd = false)
+{
+	string fieldValue;
+	string::size_type startPos = string::npos;
+
+	if (start.empty() == true)
+	{
+		startPos = 0;
+	}
+	else
+	{
+		startPos = str.find(start, endPos);
+	}
+
+	if (startPos != string::npos)
+	{
+		startPos += start.length();
+
+		if (end.empty() == true)
+		{
+			fieldValue = str.substr(startPos);
+		}
+		else
+		{
+			if (anyCharacterOfEnd == false)
+			{
+				endPos = str.find(end, startPos);
+			}
+			else
+			{
+				endPos = str.find_first_of(end, startPos);
+			}
+			if (endPos != string::npos)
+			{
+				fieldValue = str.substr(startPos, endPos - startPos);
+			}
+		}
+	}
+
+	return fieldValue;
+}
+
 GMimeMboxFilter::GMimeMboxPart::GMimeMboxPart(const string &subject,
 	dstring &buffer) :
 	m_subject(subject),
@@ -117,6 +156,8 @@ GMimeMboxFilter::GMimeMboxFilter(const string &mime_type) :
 	m_pMimeMessage(NULL),
 	m_partsCount(-1),
 	m_partNum(-1),
+	m_partLevel(-1),
+	m_currentLevel(0),
 	m_messageStart(0),
 	m_foundDocument(false)
 {
@@ -172,7 +213,8 @@ bool GMimeMboxFilter::set_document_data(const char *data_ptr, unsigned int data_
 {
 	// Close/free whatever was opened/allocated on a previous call to set_document()
 	finalize(true);
-	m_partsCount = m_partNum = -1;
+	m_partsCount = m_partNum = m_partLevel = -1;
+	m_levels.clear();
 	m_messageStart = 0;
 	m_messageDate.clear();
 	m_partCharset.clear();
@@ -200,7 +242,8 @@ bool GMimeMboxFilter::set_document_file(const string &file_path, bool unlink_whe
 {
 	// Close/free whatever was opened/allocated on a previous call to set_document()
 	finalize(true);
-	m_partsCount = m_partNum = -1;
+	m_partsCount = m_partNum = m_partLevel = -1;
+	m_levels.clear();
 	m_messageStart = 0;
 	m_messageDate.clear();
 	m_partCharset.clear();
@@ -255,14 +298,42 @@ bool GMimeMboxFilter::skip_to_document(const string &ipath)
 		return true;
 	}
 
-	// ipath's format is "o=offset&p=part_number"
-	if (sscanf(ipath.c_str(), "o=%u&p=%d", (unsigned int*)&m_messageStart, &m_partNum) != 2)
+	// ipath's format is "o=offset&l=part_levels"
+	if (sscanf(ipath.c_str(), "o=%u&l=[", (unsigned int*)&m_messageStart) != 1)
 	{
 		return false;
 	}
 
 	finalize(false);
 	m_partsCount = -1;
+	m_levels.clear();
+	string::size_type levelsPos = ipath.find("l=[");
+	if (levelsPos != string::npos)
+	{
+		string::size_type endPos = 0;
+		string levels(ipath.substr(levelsPos + 2));
+		string levelInfo(extractField(levels, "[", "]", endPos));
+
+		// Parse levels
+		while (levelInfo.empty() == false)
+		{
+			int partLevel = 0, partsCount = 0, partNum = 0;
+
+#ifdef DEBUG
+			clog << "GMimeMboxFilter::skip_to_document: level " << levelInfo << endl;
+#endif
+			if (sscanf(levelInfo.c_str(), "%d,%d,%d", &partLevel, &partsCount, &partNum) == 3)
+			{
+				m_levels[partLevel] = pair<int, int>(partsCount, partNum);
+			}
+
+			if (endPos == string::npos)
+			{
+				break;
+			}
+			levelInfo = extractField(levels, "[", "]", endPos);
+		}
+	}
 	m_messageDate.clear();
 	m_partCharset.clear();
 	m_foundDocument = false;
@@ -310,7 +381,7 @@ int GMimeMboxFilter::openFile(const string &filePath)
 	if (fd < 0)
 	{
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::openFile: couldn't open " << filePath << endl;
+		clog << "GMimeMboxFilter::openFile: couldn't open " << filePath << endl;
 #endif
 		return false;
 	}
@@ -336,7 +407,7 @@ bool GMimeMboxFilter::initializeData(void)
 		ssize_t streamLength = g_mime_stream_length(m_pGMimeMboxStream);
 
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::initializeData: from offset " << m_messageStart
+		clog << "GMimeMboxFilter::initializeData: from offset " << m_messageStart
 			<< " to " << streamLength << endl;
 #endif
 		if (m_messageStart > (GMIME_OFFSET_TYPE)streamLength)
@@ -376,7 +447,7 @@ bool GMimeMboxFilter::initializeFile(void)
 		m_pGMimeMboxStream = g_mime_stream_fs_new_with_bounds(m_fd, m_messageStart, (GMIME_OFFSET_TYPE)streamLength);
 #endif
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::initializeFile: stream starts at offset " << m_messageStart << endl;
+		clog << "GMimeMboxFilter::initializeFile: stream starts at offset " << m_messageStart << endl;
 #endif
 	}
 	else
@@ -410,7 +481,7 @@ bool GMimeMboxFilter::initialize(void)
 		return true;
 	}
 #ifdef DEBUG
-	cout << "GMimeMboxFilter::initialize: couldn't create new parser" << endl;
+	clog << "GMimeMboxFilter::initialize: couldn't create new parser" << endl;
 #endif
 
 	return false;
@@ -472,7 +543,7 @@ bool GMimeMboxFilter::readStream(GMimeStream *pStream, dstring &fileBuffer)
 	bool gotOutput = true;
 
 #ifdef DEBUG
-	cout << "GMimeMboxFilter::readStream: stream is " << streamLen
+	clog << "GMimeMboxFilter::readStream: stream is " << streamLen
 		<< " bytes long" << endl;
 #endif
 
@@ -482,7 +553,7 @@ bool GMimeMboxFilter::readStream(GMimeStream *pStream, dstring &fileBuffer)
 			(totalSize >= m_maxSize))
 		{
 #ifdef DEBUG
-			cout << "GMimeMboxFilter::readStream: stopping at "
+			clog << "GMimeMboxFilter::readStream: stopping at "
 				<< totalSize << endl;
 #endif
 			break;
@@ -508,7 +579,7 @@ bool GMimeMboxFilter::readStream(GMimeStream *pStream, dstring &fileBuffer)
 		}
 	} while (bytesRead > 0);
 #ifdef DEBUG
-	cout << "GMimeMboxFilter::readStream: read " << totalSize
+	clog << "GMimeMboxFilter::readStream: read " << totalSize
 		<< "/" << fileBuffer.size() << " bytes" << endl;
 #endif
 
@@ -529,21 +600,37 @@ bool GMimeMboxFilter::nextPart(const string &subject)
 			m_content.clear();
 			if (extractPart(pMimePart, mboxPart) == true)
 			{
+				string ipath;
 				char posStr[128];
 
 				// New document
 				m_metaData.clear();
 				m_metaData["title"] = mboxPart.m_subject;
 				m_metaData["mimetype"] = mboxPart.m_contentType;
-				m_metaData["date"] = m_messageDate;
+				if (m_messageDate.empty() == false)
+				{
+					m_metaData["date"] = m_messageDate;
+				}
 				m_metaData["charset"] = m_partCharset;
 				snprintf(posStr, 128, "%u", m_content.length());
 				m_metaData["size"] = posStr;
-				// FIXME: use the same scheme as Mozilla
-				snprintf(posStr, 128, "o=%u&p=%d", m_messageStart, max(m_partNum - 1, 0));
-				m_metaData["ipath"] = posStr;
+				snprintf(posStr, 128, "o=%u&l=", m_messageStart);
+				ipath = posStr;
+				for (map<int, pair<int, int> >::const_iterator levelIter = m_levels.begin();
+					levelIter != m_levels.end(); ++levelIter)
+				{
+					int partNum = levelIter->second.second;
+
+					if (levelIter->first == m_partLevel)
+					{
+						partNum = m_partNum;
+					}
+					snprintf(posStr, 128, "[%d,%d,%d]", levelIter->first, levelIter->second.first, partNum);
+					ipath += posStr;
+				}
+				m_metaData["ipath"] = ipath;
 #ifdef DEBUG
-				cout << "GMimeMboxFilter::nextPart: message location is " << posStr << endl; 
+				clog << "GMimeMboxFilter::nextPart: message location is " << ipath << endl; 
 #endif
 
 #ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
@@ -570,7 +657,7 @@ bool GMimeMboxFilter::nextPart(const string &subject)
 	}
 
 	// If we get there, no suitable parts were found
-	m_partsCount = m_partNum = -1;
+	m_partsCount = m_partNum = m_partLevel = -1;
 
 	return false;
 }
@@ -586,7 +673,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	while (GMIME_IS_MESSAGE_PART(part))
 	{
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::extractPart: nested message part" << endl;
+		clog << "GMimeMboxFilter::extractPart: nested message part" << endl;
 #endif
 		GMimeMessage *partMessage = g_mime_message_part_get_message(GMIME_MESSAGE_PART(part));
 		part = g_mime_message_get_mime_part(partMessage);
@@ -598,46 +685,105 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	// Is this a multipart ?
 	if (GMIME_IS_MULTIPART(part))
 	{
+		int partNum = 0;
+		bool gotPart = false;
+
 #ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 		m_partsCount = g_mime_multipart_get_count(GMIME_MULTIPART(part));
 #else
 		m_partsCount = g_mime_multipart_get_number(GMIME_MULTIPART(part));
 #endif
+		++m_currentLevel;
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::extractPart: message has " << m_partsCount << " parts" << endl;
+		clog << "GMimeMboxFilter::extractPart: message has " << m_partsCount << " parts at level " << m_currentLevel << endl;
 #endif
-		for (int partNum = max(m_partNum, 0); partNum < m_partsCount; ++partNum)
+
+		map<int, pair<int, int> >::iterator levelIter = m_levels.find(m_currentLevel);
+		if (levelIter != m_levels.end())
+		{
+			pair<int, int> partPair = levelIter->second;
+#ifdef DEBUG
+			clog << "GMimeMboxFilter::extractPart: level " << m_currentLevel << " had " << partPair.first << " parts" << endl;
+#endif
+			if (partPair.first == m_partsCount)
+			{
+				partNum = partPair.second;
+#ifdef DEBUG
+				clog << "GMimeMboxFilter::extractPart: restarting level " << m_currentLevel << " at part " << partNum << endl;
+#endif
+			}
+		}
+		else
+		{
+			partNum = 0;
+		}
+
+		for (; partNum < m_partsCount; ++partNum)
 		{
 #ifdef DEBUG
-			cout << "GMimeMboxFilter::extractPart: extracting part " << partNum << endl;
+			clog << "GMimeMboxFilter::extractPart: extracting part " << partNum << endl;
 #endif
-			
+			m_partNum = partNum;
+
 			GMimeObject *multiMimePart = g_mime_multipart_get_part(GMIME_MULTIPART(part), partNum);
 			if (multiMimePart == NULL)
 			{
 				continue;
 			}
 
-			bool gotPart = extractPart(multiMimePart, mboxPart);
+			gotPart = extractPart(multiMimePart, mboxPart);
 #ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
 			g_mime_object_unref(multiMimePart);
 #endif
 
 			if (gotPart == true)
 			{
-				m_partNum = ++partNum;
-				return true;
+				break;
 			}
 		}
 
+		if (gotPart == true)
+		{
+			// Were all parts in the next level parsed ?
+			map<int, pair<int, int> >::iterator levelIter = m_levels.find(m_currentLevel + 1);
+			if ((levelIter == m_levels.end()) ||
+				(levelIter->second.second + 1 >= levelIter->second.first))
+			{
+				// Move to the next part at this level
+				++partNum;
+			}
+
+			levelIter = m_levels.find(m_currentLevel);
+			if (levelIter != m_levels.end())
+			{
+				if (partNum > levelIter->second.second)
+				{
+					levelIter->second.second = partNum;
+#ifdef DEBUG
+					clog << "GMimeMboxFilter::extractPart: remembering to restart level " << m_currentLevel << " at part " << partNum << endl;
+#endif
+				}
+			}
+			else
+			{
+				m_levels[m_currentLevel] = pair<int, int>(m_partsCount, partNum);
+#ifdef DEBUG
+				clog << "GMimeMboxFilter::extractPart: remembering to restart level " << m_currentLevel << " at part " << partNum << endl;
+#endif
+			}
+			--m_currentLevel;
+
+			return true;
+		}
+
 		// None of the parts were suitable
-		m_partsCount = m_partNum = -1;
+		m_partsCount = m_partNum = m_partLevel = -1;
 	}
 
 	if (!GMIME_IS_PART(part))
 	{
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::extractPart: not a part" << endl;
+		clog << "GMimeMboxFilter::extractPart: not a part" << endl;
 #endif
 		return false;
 	}
@@ -654,7 +800,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	if (partType != NULL)
 	{
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::extractPart: type is " << partType << endl;
+		clog << "GMimeMboxFilter::extractPart: type is " << partType << endl;
 #endif
 		mboxPart.m_contentType = partType;
 
@@ -667,7 +813,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 				string contentAccessType(partAccessType);
 
 #ifdef DEBUG
-				cout << "GMimeMboxFilter::extractPart: part access type is " << contentAccessType << endl;
+				clog << "GMimeMboxFilter::extractPart: part access type is " << contentAccessType << endl;
 #endif
 				if (contentAccessType == "local-file")
 				{
@@ -679,7 +825,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 						mboxPart.m_subject = partLocalFile;
 						mboxPart.m_buffer.clear();
 #ifdef DEBUG
-						cout << "GMimeMboxFilter::extractPart: local file at " << partLocalFile << endl;
+						clog << "GMimeMboxFilter::extractPart: local file at " << partLocalFile << endl;
 #endif
 
 						// Load the part from file
@@ -702,7 +848,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 				{
 					mboxPart.m_contentType = "application/octet-stream";
 #ifdef DEBUG
-					cout << "GMimeMboxFilter::extractPart: unknown part access type" << endl;
+					clog << "GMimeMboxFilter::extractPart: unknown part access type" << endl;
 #endif
 				}
 			}
@@ -722,7 +868,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	GMimePartEncodingType encodingType = g_mime_part_get_encoding(mimePart);
 #endif
 #ifdef DEBUG
-	cout << "GMimeMboxFilter::extractPart: encoding is " << encodingType << endl;
+	clog << "GMimeMboxFilter::extractPart: encoding is " << encodingType << endl;
 #endif
 #ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 	g_mime_part_set_content_encoding(mimePart, GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
@@ -733,7 +879,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	if (fileName != NULL)
 	{
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::extractPart: file name is " << fileName << endl;
+		clog << "GMimeMboxFilter::extractPart: file name is " << fileName << endl;
 #endif
 		mboxPart.m_subject = fileName;
 	}
@@ -757,7 +903,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 			if (charsetFilter != NULL)
 			{
 #ifdef DEBUG
-				cout << "GMimeMboxFilter::extractPart: converting from charset " << charset << endl;
+				clog << "GMimeMboxFilter::extractPart: converting from charset " << charset << endl;
 #endif
 				g_mime_stream_filter_add(GMIME_STREAM_FILTER(memStream), charsetFilter);
 				g_object_unref(charsetFilter);
@@ -772,7 +918,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	{
 		ssize_t writeLen = g_mime_data_wrapper_write_to_stream(dataWrapper, memStream);
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::extractPart: wrote " << writeLen << " bytes" << endl;
+		clog << "GMimeMboxFilter::extractPart: wrote " << writeLen << " bytes" << endl;
 #endif
 		if (G_IS_OBJECT(dataWrapper))
 		{
@@ -801,6 +947,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	{
 		g_object_unref(memStream);
 	}
+	m_partLevel = m_currentLevel;
 
 	return true;
 }
@@ -809,6 +956,7 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 {
 	string msgSubject(subject);
 
+	m_currentLevel = 0;
 	while (g_mime_stream_eos(m_pGMimeMboxStream) == FALSE)
 	{
 		// Does the previous message have parts left to parse ?
@@ -832,7 +980,7 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 			m_pMimeMessage = g_mime_parser_construct_message(m_pParser);
 			if (m_pMimeMessage == NULL)
 			{
-				cerr << "Couldn't construct new MIME message" << endl;
+				clog << "Couldn't construct new MIME message" << endl;
 				break;
 			}
 
@@ -843,7 +991,7 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 			off_t messageEnd = g_mime_parser_tell(m_pParser);
 #endif
 #ifdef DEBUG
-			cout << "GMimeMboxFilter::extractMessage: message between offsets " << m_messageStart
+			clog << "GMimeMboxFilter::extractMessage: message between offsets " << m_messageStart
 				<< " and " << messageEnd << endl;
 #endif
 			if (messageEnd > m_messageStart)
@@ -861,7 +1009,7 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 						(mozFlags & 0x0040))
 					{
 #ifdef DEBUG
-						cout << "GMimeMboxFilter::extractMessage: flagged by Mozilla" << endl;
+						clog << "GMimeMboxFilter::extractMessage: flagged by Mozilla" << endl;
 #endif
 						continue;
 					}
@@ -883,7 +1031,7 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 						if (evoFlags & 0x0002)
 						{
 #ifdef DEBUG
-							cout << "GMimeMboxFilter::extractMessage: flagged by Evolution" << endl;
+							clog << "GMimeMboxFilter::extractMessage: flagged by Evolution" << endl;
 #endif
 							continue;
 						}
@@ -892,6 +1040,10 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 
 				// How old is this message ?
 				const char *pDate = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), "Date");
+				if (pDate == NULL)
+				{
+					pDate = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), "Resent-Date");
+				}
 				if (pDate != NULL)
 				{
 					m_messageDate = pDate;
@@ -910,7 +1062,13 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 					{
 						char timeStr[64];
 
+						// FIXME: don't use this extension ?
+#if defined(__GNU_LIBRARY__)
+						// %z is a GNU extension
+						if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %z", pTimeTm) > 0)
+#else
 						if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %Z", pTimeTm) > 0)
+#endif
 						{
 							m_messageDate = timeStr;
 						}
@@ -919,7 +1077,7 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 					delete pTimeTm;
 				}
 #ifdef DEBUG
-				cout << "GMimeMboxFilter::extractMessage: message date is " << m_messageDate << endl;
+				clog << "GMimeMboxFilter::extractMessage: message date is " << m_messageDate << endl;
 #endif
 
 				// Extract the subject
@@ -931,7 +1089,7 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 			}
 		}
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::extractMessage: message subject is " << msgSubject << endl;
+		clog << "GMimeMboxFilter::extractMessage: message subject is " << msgSubject << endl;
 #endif
 
 		if (nextPart(msgSubject) == true)
